###Part I: Preparing the data
rm(list=ls())

##Set your working directory manually or using the following commands## 
#getwd()
#setwd("pathway")
setwd("C:/Users/Pedro/Documents/Classes/Methods in Ecology/2013 fall") # May have to adapt that


## read the Hypericum survival data from file
orig_data <- read.table("Hypericum_survival.txt", header=T)
attach(orig_data)
names(orig_data)

## change variables to include only reproductive individuals
survival <- survival[rep_structures>0]
rep_structures <- rep_structures[rep_structures>0]

###Part II: Logistic regression models

## construct a logistic regression model of survival vs. reproductive structures
model1 <- glm(survival ~ rep_structures, family=binomial)
summary(model1)

## Calculate the G^2 statistic from the deviance values
G_sq <- model1$null.deviance - model1$deviance
G_sq
## now, get the P-value by comparing to the Chi-square distribution
pchisq(G_sq, 1, lower.tail=F)

## get fitted values for model1 and plot
par(mfrow=c(1,2))

#in the natural scale
x_values_rs <- seq(0, max(rep_structures), 1)
y_values_rs <- predict(model1, list(rep_structures=x_values_rs), type="response")
plot(rep_structures, survival,type="n",ylab="P(survival)")
rug(jitter(rep_structures[survival==0]),ticksize = 0.03,)
rug(jitter(rep_structures[survival==1]),ticksize = 0.03, side=3)
lines(x_values_rs, y_values_rs)
b <- seq(0, max(rep_structures),30)
z <- cut(rep_structures, b)
prebyden <- tapply(survival,z,sum)
tab <- table(z)
probs <-prebyden/tab
probs <- as.vector(probs)
points(b[2:length(b)],probs,pch=16,cex=1)
se2<-sqrt(probs*(1-probs)/tab)
up2 <-probs+as.vector(se2)
down2 <-probs-as.vector(se2)
for (i in 1:length(b-1)){
lines(c(b[i+1],b[i+1]),c(up2[i],down2[i]))}

#in the logit scale
coef <-model1$coefficients
plot(b[2:length(b)],log(probs/(1-probs)),xlab="# reproductive structures")
abline(a=coef[1],b=coef[2])

##use model to predict the survival probability of a plant with 500 reproductive structures
      rs <- 500
      coeffs <- summary(model1)$coefficients
      odds_ratio <- coeffs[1,1]  + coeffs[2,1]*rs
      prob <- 1/(1 + (1/exp(odds_ratio)))
      prob

##model with log-transformed predictor variable
rep <- log(rep_structures)
model2 <-glm(survival ~ rep, family=binomial)
summary(model2)

## Calculate the G^2 statistic from the deviance values
G_sq <- model2$null.deviance - model2$deviance
G_sq
## now, get the P-value by comparing to the Chi-square distribution
pchisq(G_sq, 1, lower.tail=F)

## get fitted values for model2 and plot
par(mfrow=c(1,2))

#in the natural scale
x_values_lrs <- seq(0, max(rep)+1, 1)
y_values_lrs <- predict(model2, list(rep=x_values_lrs), type="response")
plot(rep, survival,type="n",ylab="P(survival)",xlab="log(# reproductive structures)")
rug(jitter(rep[survival==0]),ticksize = 0.03,)
rug(jitter(rep[survival==1]),ticksize = 0.03, side=3)
lines(x_values_lrs, y_values_lrs)
b <- seq(0, max(rep),0.5)
z <- cut(rep, b)
prebyden <- tapply(survival,z,sum)
tab <- table(z)
probs <-prebyden/tab
probs <- as.vector(probs)
points(b[2:length(b)],probs,pch=16,cex=1)
se2<-sqrt(probs*(1-probs)/tab)
up2 <-probs+as.vector(se2)
down2 <-probs-as.vector(se2)
for (i in 1:length(b-1)){
 lines(c(b[i+1],b[i+1]),c(up2[i],down2[i]))}

#in the logit scale
coef <-model2$coefficients
plot(b[2:length(b)],log(probs/(1-probs)),xlab="log(reproductive structures)")
abline(a=coef[1],b=coef[2])

##Plot models 1 (non-tranformed) and 2 (log-transformed) side by side
par(mfrow=c(1,1))
plot(rep_structures, survival,type="n",ylab="P(survival)",xlab="# reproductive structures")
rug(jitter(rep_structures[survival==0]),ticksize = 0.03,)
rug(jitter(rep_structures[survival==1]),ticksize = 0.03, side=3)
x_values_rs <- seq(0, max(rep_structures),1)
y_values_rs <- predict(model1, list(rep_structures=x_values_rs), type="response")
lines(x_values_rs, y_values_rs, col="blue")
x_values_lrs <- seq(0, max(rep), 0.1)
y_values_lrs <- predict(model2, list(rep=x_values_lrs), type="response")
lines(exp(x_values_lrs), y_values_lrs, col="red")
b <- seq(0, max(rep_structures),30)
z <- cut(rep_structures, b)
prebyden <- tapply(survival,z,sum)
tab <- table(z)
probs <-prebyden/tab
probs <- as.vector(probs)
points(b[2:length(b)],probs,pch=16,cex=1)
se2<-sqrt(probs*(1-probs)/tab)
up2 <-probs+as.vector(se2)
down2 <-probs-as.vector(se2)
for (i in 1:length(b-1)){
 lines(c(b[i+1],b[i+1]),c(up2[i],down2[i]))}

##use model to predict the survival probability of a plant with 500 reproductive structures
      rs <- log(500)
      coeffs <- summary(model2)$coefficients
      odds_ratio <- coeffs[1,1]  + coeffs[2,1]*rs
      prob <- 1/(1 + (1/exp(odds_ratio)))
      prob

## compute AIC for the models
AIC(model1,model2)

## use the predict function to check our result
new_data <- data.frame(list(rep=log(500)))
predict(model2, newdata=new_data, type="response")

###Part III: Logistic regression with a Bayesian approach 

##non-transformed (model 1)
library(R2OpenBUGS)
n <- NROW(rep_structures)
x <- rep_structures
y <-survival
  
# Write model
Logmodel<-function()
 {
  for( i in 1 : n ) {
    y[i] ~ dbern( mu[i] )
    mu[i] <- 1/(1+exp(-( b0 + inprod( b1 , x[i] ))))
                    }
  b0 ~ dnorm( 0 , 1.0E-12 )
  b1 ~ dnorm( 0 , 1.0E-12 )
      prob_est <- 1/(1 + (1/exp(b0 + b1*500)))
      }
write.model(Logmodel, "Logmodel.txt")

# Bundle data
win.data <- list("x","y","n")

# Inits function
inits <- function(){ list(b0=0.1, b1=0.1)}

# Parameters to estimate
params <- c("b0","b1","prob_est")

# MCMC settings
nc = 3 
ni=10000
nb=1000
nt=1

# Start Gibbs sampler
out <- bugs(data = win.data, inits = inits, parameters = params, model = "logmodel.txt",
n.thin = nt, n.chains = nc, n.burnin = nb, n.iter = ni, codaPkg=T)

library(coda)
reg.coda1<-read.bugs(out)
results1<-summary(reg.coda1)
results1

##log-transformed (model2) 
library(R2OpenBUGS)
n <- NROW(rep)
x <- rep
y <-survival
  
# Write model
Logmodel<-function()
 {
  for( i in 1 : n ) {
    y[i] ~ dbern( mu[i] )
    mu[i] <- 1/(1+exp(-( b0 + inprod( b1 , x[i] ))))
                    }
  b0 ~ dnorm( 0 , 1.0E-12 )
  b1 ~ dnorm( 0 , 1.0E-12 )
      prob_est <- 1/(1 + (1/exp(b0 + b1*log(500))))
 
      }
write.model(Logmodel, "Logmodel.txt")

# Bundle data
win.data <- list("x","y","n")

# Inits function
inits <- function(){ list(b0=0.1, b1=0.1)}

# Parameters to estimate
params <- c("b0","b1","prob_est")

# MCMC settings
nc = 3 
ni=10000
nb=1000
nt=1

# Start Gibbs sampler
out <- bugs(data = win.data, inits = inits, parameters = params, model = "logmodel.txt",
n.thin = nt, n.chains = nc, n.burnin = nb, n.iter = ni, codaPkg=T)

library(coda)
reg.coda2<-read.bugs(out)
results2<-summary(reg.coda2)
results2

##Plot the Bayesian credibility intervals together with the frequentist models
estim1 <- estim2 <- array(0,c(140,100))
x <- seq(1,1400,10)
s1 <- round(runif(100)*5000,0)
s2 <- round(runif(100)*5000,0)
for(i in 1:100){
for(j in 1:140){
estim1[j,i] <- 1/(1 + (1/exp(as.numeric(reg.coda1[s1[i],1][1]) + as.numeric(reg.coda1[s1[i],2][1])*x[j])))
estim2[j,i] <- 1/(1 + (1/exp(as.numeric(reg.coda2[s2[i],1][1]) + as.numeric(reg.coda2[s2[i],2][1])*log(x[j]))))
                }
                        }
plot(x_values_rs, y_values_rs, col="blue", type="l",xlab ="Number of reproductive structures", ylab ="Probability")
for (i in 1:100){
lines(exp(log(seq(1,1400,10))),estim1[,i],type="l",col="gray")
lines(seq(1,1400,10),estim2[,i],type="l",col="cyan")
}
points(exp(x_values_lrs), y_values_lrs, col="red",type="l")
points(x_values_rs, y_values_rs, col="blue", type="l")
b <- seq(0, max(rep_structures),30)
z <- cut(rep_structures, b)
prebyden <- tapply(survival,z,sum)
tab <- table(z)
probs <-prebyden/tab
probs <- as.vector(probs)
points(b[2:length(b)],probs,pch=16,cex=1)
se2<-sqrt(probs*(1-probs)/tab)
up2 <-probs+as.vector(se2)
down2 <-probs-as.vector(se2)
for (i in 1:length(b-1)){
 lines(c(b[i+1],b[i+1]),c(up2[i],down2[i]))}

## detach the data
detach(orig_data)

